#include <QProcess>
#include <QStringList>
#include <QUrl>

#include <QDebug>

#include "logger.h"
#include "ui/browser/browser.h"
#include "ui/widgets/document.h"
#include "ui/widgets/image.h"
#include "ui/widgets/pdf.h"
#include "ui/widgets/video.h"
#include "ui/widgets/webpage.h"
#include "playlistparser.h"
#include "downloader.h"
#include "dispatcher.h"

Dispatcher::Dispatcher(Viewer *viewer, QString path)
    : QObject()
    , m_status(Dispatcher::StatusNone)
    , m_currentWidget(0)
    , m_prevWidget(0)
    , m_viewer(viewer)
    , m_loopIdx(-1)
{
    settings = Settings::load();
    m_mode = settings->get("main/mode").toString();

    connect(viewer, &Viewer::end, this, &Dispatcher::next);
    connect(viewer, &Viewer::mouseEvent, this, &Dispatcher::onMouseEvent);

    if (settings->get("main/no_screensaver").toBool())
        disableScreensaver();
}

void Dispatcher::StartSession()
{
    if ((m_status == Dispatcher::StatusBrowser) ||
            (m_status == Dispatcher::StatusAfterBrowser))
        return;

    m_currentWidget->stop();
    startBrowser();

}

void Dispatcher::StopSession()
{
    if (m_status != Dispatcher::StatusBrowser)
        return;

    m_currentWidget->stop();
    startSlideshow();
}

void Dispatcher::AddTime(int seconds)
{
    // TODO
}

void Dispatcher::onScreensaverTimeout()
{
    QProcess p;
    QStringList args;
    args << "-deactivate";
    p.start("xscreensaver-command", args);
    p.waitForFinished();
}

void Dispatcher::disableScreensaver()
{
    connect(&m_ssTimer, &QTimer::timeout, this, &Dispatcher::onScreensaverTimeout);
    m_ssTimer.start(30000);
}

void Dispatcher::startup()
{
    if ((m_mode == "kiosk") &&
            settings->get("main/start_with_session").toBool())
        startBrowser();
    else
        startSlideshow();
}

void Dispatcher::onMouseEvent()
{
    if (m_mode != "kiosk")
        return;

    // TODO: tell the viewer to stop working
    startBrowser();
}

void Dispatcher::onReconnectMouseEventTimeout()
{
    connect(m_viewer, &Viewer::mouseEvent, this, &Dispatcher::onMouseEvent);
}

void Dispatcher::reconnectMouseEvent()
{
    QTimer::singleShot(1000, this, "onReconnectMouseEventTimeout");
}

void Dispatcher::next()
{
    if (m_status == Dispatcher::StatusNone)
    {
        startup();
    }

    else if (m_status == Dispatcher::StatusBrowser)
    {
        qDebug() << "dispatcher: End StatusBrowser";
        disconnect(m_viewer, &Viewer::mouseEvent, this, &Dispatcher::onMouseEvent);
        afterBrowser();
    }

    else if (m_status == Dispatcher::StatusAfterBrowser)
    {
        qDebug() << "dispatcher: End StatusAfterBrowser";
        execHook("session-stop");
        dynamic_cast<Browser *>(m_currentWidget)->cleanAll();
        Logger::logInfo("STARTING: web session");
        if (settings->get("main/after_web_session").toString() == "session")
            startBrowser();
        else
        {
            reconnectMouseEvent();
            startSlideshow();
        }
    }

    else
    {
        showLoopNext();
    }
}

void Dispatcher::afterBrowser()
{
    m_status = Dispatcher::StatusAfterBrowser;
    QString uri = settings->get("main/post_session_uri").toString();
    if (uri.isEmpty())
    {
        next();
        return;
    }
    int time = settings->get("main/post_session_uri_time").toInt();
    PlaylistItem item(uri, PlaylistItem::TypeWebpage, time);
    m_currentWidget = new Webpage(&item, m_viewer);
    m_viewer->next(dynamic_cast<QWidget *>(m_currentWidget));
}

void Dispatcher::startBrowser()
{
    execHook("session-start");
    Logger::logInfo("STARTING: web session");
    m_status = Dispatcher::StatusBrowser;
    m_currentWidget = new Browser;
    m_viewer->next(dynamic_cast<QWidget *>(m_currentWidget));
}

void Dispatcher::execHook(QString hook)
{
    QString hookFile = QString("/etc/xdg/webkiosk/hooks/%1").arg(hook);
    QProcess p;
    p.start(hookFile);
    p.waitForFinished();
}

void Dispatcher::startSlideshow()
{
    QUrl url;

    if (settings->get("slideshow/use_remote").toBool())
        url = QUrl(settings->get("slideshow/pages").toString());
    else
        url = QUrl(QString("file://%1").arg(settings->playlistFile()));

    PlaylistParser *pl = PlaylistParser::get(url);
    m_loop = pl->selectLoop();
    if (m_loop->downloadable())
    {
        qDebug() << "DOWNLOADABLE";
        Downloader *d = new Downloader(m_loop);
        d->start();
        connect(d, &Downloader::downloadDone, this, &Dispatcher::showLoopNext);
    }
    else
    {
        qDebug() << "Not downloadable";
        showLoopNext();
    }
}

void Dispatcher::showLoopNext()
{
    m_status = Dispatcher::StatusPlaylist;
    ++m_loopIdx;
    qDebug() << "ID:" << m_loopIdx;
    if (m_loopIdx >= m_loop->count())
        m_loopIdx = 0;

    PlaylistItem *item = m_loop->at(m_loopIdx);

    qDebug() << item->uri() << item->type() << item->href();

    if (item->href())
    {
        showLoopNext();
        return;
    }

    m_prevWidget = m_currentWidget;

    // TODO: something nicer (map?)
    switch (item->type())
    {
        case PlaylistItem::TypeImage:
            m_currentWidget = new Image(item, m_viewer);
            break;
        case PlaylistItem::TypeVideo:
            m_currentWidget = new Video(item, m_viewer);
            break;
        case PlaylistItem::TypePdf:
            m_currentWidget = new PDF(item, m_viewer);
            break;
        case PlaylistItem::TypeWebpage:
            m_currentWidget = new Webpage(item, m_viewer);
            break;
        case PlaylistItem::TypeDocument:
            m_currentWidget = new Document(item, m_viewer);
            break;
        default:
            next();
            return;
    }

    m_viewer->next(dynamic_cast<QWidget *>(m_currentWidget));
    if (m_prevWidget)
        dynamic_cast<QWidget *>(m_prevWidget)->deleteLater();
}
