#include <QUrl>
#include <QDir>

#include <QDebug>

#include "downloader.h"

Downloader::Downloader(PlaylistLoop *loop)
    : QObject()
    , m_loop(loop)
    , m_idl(0)
    , m_index(-1)
{
    m_baseUrl = m_loop->baseUrl();
    m_cacheDir = QDir::homePath() + "/.linutop-kiosk";
    m_dataDir = m_cacheDir + "/data";
    m_etagDir = m_cacheDir + "/etag";

    QDir d;
    d.mkpath(m_dataDir);
    d.mkpath(m_etagDir);
}

Downloader::~Downloader()
{
}

void Downloader::start()
{
    downloadNext();
}

QString Downloader::localEtag(QString url)
{
    QFileInfo info(url);
    QString etagFile = m_etagDir + "/" + info.fileName();

    QFile f(etagFile);
    if(!f.open(QIODevice::ReadOnly))
        return QString();

    QTextStream in(&f);
    QString etag = in.readLine();
    f.close();

    return etag;
}

void Downloader::writeEtag(QString url, int d, int s)
{
    QFileInfo info(url);
    QString etagFile = m_etagDir + "/" + info.fileName();

    QFile f(etagFile);
    if(!f.open(QIODevice::WriteOnly))
        return;

    QTextStream out(&f);
    out << d << ":" << s;
    f.close();
}

bool Downloader::checkEtag (QString url, int d, int s)
{
    QString toCheck = QString("%1:%2").arg(d, s);

    QString local = localEtag(url);
    if (local.isEmpty())
        return false;

    if (local == toCheck)
        return true;

    return false;
}

void Downloader::downloadNext()
{
    qDebug() << "NEXT Download";
    if (m_idl)
    {
        delete m_idl;
        m_idl = 0;
    }

    ++m_index;

    qDebug() << "DL idx:" << m_index << "/" << m_loop->count() - 1;
    if (m_index >= m_loop->count())
    {
        emit(downloadDone());
        deleteLater();
        return;
    }

    PlaylistItem *item = m_loop->at(m_index);
    qDebug() << item->uri();

    // check if we need to download the data
    if (item->type() == PlaylistItem::TypeWebpage)
        downloadNext();

    QString itemUri = item->uri();
    if (itemUri.startsWith("file://"))
        downloadNext();

    if (itemUri.startsWith("rtsp://"))
        downloadNext();

    if (itemUri.at(0) == '/')
        itemUri = m_baseUrl + "/r" + itemUri;

    QFileInfo info(itemUri);
    QString localPath = m_dataDir + "/" + info.fileName();
    QString localUri = QString("file://") + localPath;
    item->setUri(localUri);

    m_idl = new ItemDownloader(itemUri, localPath);
    connect(m_idl, SIGNAL(finished()), this, SLOT(downloadNext()));
    m_idl->start();
}


ItemDownloader::ItemDownloader(QString uri, QString dest)
    : QObject()
    , m_uri(uri)
{
    m_file.setFileName(dest);
    m_nam = new QNetworkAccessManager();
}

void ItemDownloader::start()
{
    if (!m_file.open(QIODevice::WriteOnly))
    {
        emit(finished());
        return;
    }

    m_request = QNetworkRequest(QUrl(m_uri));
    m_reply = m_nam->get(m_request);

    connect(m_reply, SIGNAL(finished()), this, SLOT(onFinished()));
    connect(m_reply, SIGNAL(readyRead()), this, SLOT(onReadyRead()));
}

ItemDownloader::~ItemDownloader()
{
    m_nam->deleteLater();
}

void ItemDownloader::onReadyRead()
{
    m_file.write(m_reply->readAll());
}

QString ItemDownloader::etag()
{
    QString dateStr(m_reply->rawHeader("Last-Modified"));
    int length = QString(m_reply->rawHeader("Content-Length")).toInt();

    return QString("%1:%2").arg(dateStr, length);
}

void ItemDownloader::onFinished()
{
    m_file.write(m_reply->readAll());
    m_file.close();
    ItemDownloader::etag();
    emit(finished());
    qDebug() << "finished download";
}
