#include <QStringList>
#include <QUrl>
#include <QFileInfo>
#include <QDirIterator>

#include <QDebug>

#include "playlistloop.h"

PlaylistLoop::PlaylistLoop(QDomNode &node, QString baseUrl)
    : m_baseUrl(baseUrl)
    , m_idle(false)
    , m_priority(0)
    , m_downloadable(true)
    , m_curIdx(0)
{
    qDebug() << "DL:" << m_downloadable;
    m_weekdays.resize(7);
    m_weekdays.fill(false);

    m_items = new QList<PlaylistItem>;

    fromXmlNode(node);
}

PlaylistLoop::PlaylistLoop(QString &data, QString baseUrl)
    : m_baseUrl(baseUrl)
    , m_idle(false)
    , m_priority(0)
    , m_downloadable(false)
    , m_curIdx(0)
{
    qDebug() << "DL:" << m_downloadable;
    m_weekdays.resize(7);
    m_weekdays.fill(false);

    m_items = new QList<PlaylistItem>;

    fromString(data);
}

PlaylistLoop::~PlaylistLoop()
{
    delete m_items;
}

void PlaylistLoop::fromString(QString &data)
{
    int lastSpace;
    int time;
    bool ok;
    QString uri;
    QStringList lines = data.split("\n");

    for (int i = 0; i < lines.count(); ++i)
    {
        QString line = lines.at(i);
        if (line.startsWith("#"))
            continue;

        lastSpace = line.lastIndexOf(' ');
        if (lastSpace < 0)
            continue;

        time = line.mid(lastSpace).toInt(&ok);
        if (!ok)
            continue;

        uri = line.left(lastSpace);

        QUrl url(uri);
        if (!url.isValid())
            continue;

        if (url.scheme() == "file")
        {
            QFileInfo info(url.path());
            if (!info.exists())
                continue;
            if (!info.isDir())
            {
                m_items->append(PlaylistItem(uri, PlaylistItem::TypeUndefined, time, false));
            }
            else
            {
                QDirIterator iterator(url.path(), QDirIterator::Subdirectories);
                while (iterator.hasNext())
                {
                    iterator.next();
                    if (iterator.fileInfo().isDir())
                        continue;

                    PlaylistItem item(QString("file://") + iterator.filePath(), PlaylistItem::TypeUndefined, time, false);
                    if (item.type() == PlaylistItem::TypeVideo)
                        item.setTime(0);
                    m_items->append(item);
                }
            }
        }
        else
        {
            m_items->append(PlaylistItem(uri, PlaylistItem::TypeUndefined, time, false));
        }

    }
}

void PlaylistLoop::fromXmlNode(QDomNode &node)
{
    QDomNode tmp;

    QDomNamedNodeMap attributes = node.attributes();
    
    tmp = attributes.namedItem("idle");
    if (!tmp.isNull())
        m_idle = (tmp.nodeValue() != "1");
    tmp = attributes.namedItem("start");
    if (!tmp.isNull())
        m_startTimeStr = tmp.nodeValue();
    tmp = attributes.namedItem("end");
    if (!tmp.isNull())
        m_endTimeStr = tmp.nodeValue();
    tmp = attributes.namedItem("weekdays");
    if (!tmp.isNull())
        m_weekdaysStr = tmp.nodeValue();
    tmp = attributes.namedItem("from");
    if (!tmp.isNull())
        m_startDateStr = tmp.nodeValue();
    tmp = attributes.namedItem("to");
    if (!tmp.isNull())
        m_endDateStr = tmp.nodeValue();
    tmp = attributes.namedItem("id");
    if (!tmp.isNull())
        m_id = tmp.nodeValue();

    if (!validate())
        return;

    QDomNodeList items = node.childNodes();
    for (int i=0; i<items.count(); ++i)
    {
        QDomNode n = items.at(i);
        PlaylistItem item = PlaylistItem::fromXmlNode(&n);
        if (item.isEmpty())
            continue;

        m_items->append(item);
    }
}

bool PlaylistLoop::validate()
{
    if (! validateDate())
        return false;
    if (! validateTime())
        return false;
    if (! validateWeekdays())
        return false;

    return true;
}

bool PlaylistLoop::validateDate()
{
    if (m_startDateStr.isEmpty())
    {
        m_priority -= 1;
        m_startDateStr = QDate::currentDate().toString("yyyy/MM/dd");
    }
    if (m_endDateStr.isEmpty())
    {
        m_priority -= 1;
        m_endDateStr = QDate(9999, 12, 31).toString("yyyy/MM/dd");
    }

    QStringList spl;
    QDate start_d, end_d;

    spl = m_startDateStr.split("/");
    if (spl.count() != 3)
        return false;
    start_d = QDate(spl.at(0).toInt(), spl.at(1).toInt(), spl.at(2).toInt());
    if (!start_d.isValid())
        return false;

    spl = m_endDateStr.split("/");
    if (spl.count() != 3)
        return false;
    end_d = QDate(spl.at(0).toInt(), spl.at(1).toInt(), spl.at(2).toInt());
    if (!end_d.isValid())
        return false;

    if (end_d <= start_d)
        return false;

    m_startDate = start_d;
    m_endDate = end_d;

    return true;
}

bool PlaylistLoop::validateTime()
{
    QStringList spl;
    int s_h, s_m, e_h, e_m;

    s_h = s_m = e_h = e_m = 0;

    spl = m_startTimeStr.split(":");
    if (spl.count() != 2)
    {
        s_h = 0;
        s_m = 0;
        m_priority -= 1;
    }
    else
    {
        s_h = spl.at(0).toInt();
        s_h = spl.at(1).toInt();
        if ((s_h < 0) || (s_h > 23) || (s_m < 0) || (s_m > 59))
        {
            s_h = 0;
            s_m = 0;
        }
    }

    spl = m_endTimeStr.split(":");
    if (spl.count() != 2)
    {
        e_h = 23;
        e_m = 59;
        m_priority -= 1;
    }
    else
    {
        e_h = spl.at(0).toInt();
        e_h = spl.at(1).toInt();
        if ((e_h < 0) || (e_h > 23) || (e_m < 0) || (e_m > 59))
        {
            e_h = 23;
            e_m = 59;
        }
    }

    if ((e_h == 0) && (e_m == 0))
    {
        e_h = 23;
        e_m = 59;
    }

    m_startTime = QTime (s_h, s_m);
    m_endTime = QTime (e_h, e_m);
    if (!m_startTime.isValid() || !m_endTime.isValid())
        return false;

    if (m_endTime < m_startTime)
        return false;

    return true;
}

bool PlaylistLoop::validateWeekdays()
{
    if (m_weekdaysStr.isEmpty())
    {
        m_priority -= 1;
        m_weekdays.fill(true);
        return true;
    }

    for (int i=0; i < m_weekdaysStr.count(); ++i)
    {
        int l = m_weekdaysStr.at(i).digitValue();
        if ((l >= 0) && (l < 7))
            m_weekdays.setBit(i, true);
    }

    return true;
}

bool PlaylistLoop::nowOk() const
{
    QDateTime now = QDateTime::currentDateTime();
    QDate today = now.date();
    int weekday = today.dayOfWeek() - 1; /* monday == 0 for us, 1 for Qt */

    if (! m_weekdays.at(weekday))
        return false;

    if ((today < m_startDate) || (today > m_endDate))
        return false;

    int now_minutes = now.time().hour() * 60 + now.time().minute();
    int start_minutes = m_startTime.hour() * 60 + m_startTime.minute();
    int end_minutes = m_endTime.hour() * 60 + m_endTime.minute();

    if ((now_minutes < start_minutes) || (now_minutes > end_minutes))
        return false;

    return true;
}
