#include <sys/utsname.h>

#include <QDebug>

#include <QNetworkReply>
#include <QLocale>
#include <QFile>
#include <QTextStream>

#include "restrictedwebpage.h"
#include "logger.h"
#include "config.h"

RestrictedWebPage::RestrictedWebPage(QWidget *parent)
    : QWebPage(parent)
{
    settings = Settings::load();
    setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
    setForwardUnsupportedContent(true);

    connect(this, SIGNAL(downloadRequested(const QNetworkRequest &)),
            this, SLOT(onDownloadRequested(const QNetworkRequest &)));
    connect(this, SIGNAL(unsupportedContent(QNetworkReply *)),
            this, SLOT(onUnsupportedContent(QNetworkReply *)));

    downloads = new QMap<QString, QString>;
}

void RestrictedWebPage::setupRegexList()
{
    QString fileName = settings->regexFile();
    QFile f(fileName);
    if (f.open(QIODevice::ReadOnly))
    {
        QTextStream in(&f);
        while (!in.atEnd())
        {
            QString line = in.readLine();
            if (settings->get("main/list_use_regex").toBool());
                line = line.replace(".", "\\.").replace("*", ".*");
            QRegExp re(line);
            if (!re.isValid())
                regexList << QRegExp(line);
        }
    }
    f.close();
}

RestrictedWebPage::~RestrictedWebPage()
{
    delete downloads;
}

bool RestrictedWebPage::urlIsAllowed(QUrl &url)
{
    QString mode = settings->get("main/list_mode").toString();
    if ((url.scheme() == "file") && (! settings->get("main/can_access_fs").toBool()))
        return false;

    if (mode == "none")
        return true;

    else if (mode == "whitelist")
    {
        foreach (const QRegExp &rx, regexList)
            if (rx.indexIn(url.toString()) != -1)
                return true;

        return false;
    }

    else // blacklist
    {
        foreach (const QRegExp &rx, regexList)
            if (rx.indexIn(url.toString()) != -1)
                return false;

        return true;
    }
}

bool RestrictedWebPage::acceptNavigationRequest(QWebFrame *frame,
                                                const QNetworkRequest &request,
                                                NavigationType type)
{
    QString msg;
    QUrl url = request.url();
    QString s = url.toString();
    if (urlIsAllowed(url))
    {
        msg = QString("ALLOWED: %1").arg(s);
        Logger::logInfo(msg);
        return true;
    }

    emit(urlDenied(s));
    msg = QString("DENIED: %1").arg(s);
    Logger::logInfo(msg);
    return false;
}

void RestrictedWebPage::onDownloadRequested(const QNetworkRequest &request)
{
    QNetworkRequest newreq(request);
    newreq.setUrl(request.url());
    QNetworkReply *reply = networkAccessManager()->get(newreq);
    onUnsupportedContent(reply);
}

void RestrictedWebPage::onUnsupportedContent(QNetworkReply *reply)
{
    QString mimetype;
    QString filename;

    foreach (const QByteArray &h, reply->rawHeaderList())
    {
        mimetype = QString(reply->rawHeader(h)).replace(";", "");
        if (mimetype.contains("application/force-download"))
            mimetype = "";
        else if (h.toLower() == "content-disposition")
        {
            QList<QByteArray> items = reply->rawHeader(h).split(';');
            foreach (const QByteArray &i, items)
            {
                QString tmp = i.trimmed();
                if (tmp.startsWith("filename="))
                {
                    filename = tmp.replace("filename=", "").replace("'", "").replace("\"", "");
                    break;
                }
            }
        }
    }

    // TODO: add_to_downloads
    emit(downloadStarted());
    connect(reply, SIGNAL(finished()), this, SLOT(onDownloadDone()));
}

void RestrictedWebPage::onDownloadDone()
{
    // TODO
    qDebug() << "Downloaded";
}

QString RestrictedWebPage::userAgentForUrl(const QUrl &url) const
{
    QString user_agent = settings->get("main/user_agent").toString();
    if (! user_agent.isEmpty())
        return user_agent;

    QLocale l;
    struct utsname sysinfo;
    uname(&sysinfo);

    user_agent = QString("Mozilla/5.0 (X11; %1; %2) AppleWebKit/%3 (KHTML, like Gecko) Version/5.0 Safari/%3 LinutopKiosk/%4")
                    .arg(sysinfo.sysname, l.name(), qWebKitVersion(), VERSION);

    return user_agent;
}
